#include <stdio.h>
#include <string.h>
#include <SDL/SDL.h>

#define WIDTH 640
#define HEIGHT 480

typedef struct {
	SDL_Surface *surface;
	int charpos[512];
	int maxpos;
} SDL_SurfFont;

int quitapp(SDL_SurfFont *font) {
	SDL_CloseSurfFont(font);
	SDL_Quit();
	exit(0);
}

int putpixel(SDL_Surface *scr, int x, int y, int c) {
        ((unsigned int *)scr->pixels)[y*scr->w+x]=c;
        return 0;
}

int getpixel(SDL_Surface *scr, int x, int y) {
	return ((unsigned int *)scr->pixels)[y*scr->w+x];
}

SDL_SurfFont *SDL_InitSurfFont(SDL_Surface *scr,char *file) {
	SDL_SurfFont *font;
	SDL_Surface *sfont;
	sfont=SDL_LoadBMP(file);
	sfont=SDL_ConvertSurface(sfont,scr->format,0);
	int x=0,i=0;
	font=(SDL_SurfFont *)malloc(sizeof(SDL_SurfFont));
	SDL_LockSurface(sfont);
	while (x<sfont->w) {
		if (getpixel(sfont,x,0)==0xff00ff) {
			font->charpos[i++]=x;
			while ((x<sfont->w) && (getpixel(sfont,x,0)==0xff00ff))
				x++;
			font->charpos[i++]=x;
		}
		x++;
	}
	font->maxpos=x-1;
	font->surface=SDL_LoadBMP(file);
	font->surface=SDL_ConvertSurface(font->surface,scr->format,0);
	SDL_UnlockSurface(sfont);
	SDL_FreeSurface(sfont);
	return font;
}

int SDL_WriteSurfFont(SDL_Surface *scr, SDL_SurfFont *font, int x, int y, char *text) {
	char *c;
	int charoffset,w;
	SDL_Rect srcrect, dstrect;
	w=0;
	srcrect.y=1;
	dstrect.y=y;
	srcrect.h=dstrect.h = font->surface->h-1;
	SDL_SetColorKey(font->surface,SDL_SRCCOLORKEY,getpixel(font->surface,0,font->surface->h-1));
	for (c=text;*c!='\0';c++) {
		charoffset=((int)(*c-33))*2+1;
		if (*c==' '||charoffset<0||charoffset>font->maxpos) {
			x+=font->charpos[2]-font->charpos[1];
			continue;
		}
		w=font->charpos[charoffset+1]-font->charpos[charoffset];
		srcrect.x=font->charpos[charoffset];
		srcrect.y=0;
		srcrect.w=w;
		srcrect.h=font->surface->h;
		dstrect.x=x;
		dstrect.y=y;
		dstrect.w=w;
		dstrect.h=font->surface->h;
		SDL_BlitSurface(font->surface,&srcrect,scr,&dstrect);
		x+=w;
	}
	return 0;
}

int SDL_CloseSurfFont(SDL_SurfFont *font) {
	SDL_FreeSurface(font->surface);
	free(font);
	return 0;
}

int main(int argc,char *argv[]) {
	SDL_Surface *scr;
	SDL_Event event;
	
	char fontname[10],*message="Hello, World!";
	if (argc>1) {
		strcpy(fontname,argv[1]);
	} else {
		fprintf(stdout,"Ussage: %s bmpfont\n",argv[0]);
	        exit(1);
	}
	
	if (SDL_Init(SDL_INIT_VIDEO)<0) {
                fprintf(stderr,"SDL_Init error: %s\n",SDL_GetError());
                exit(2);
        }
        atexit(SDL_Quit);
        if ((scr=SDL_SetVideoMode(WIDTH,HEIGHT,32,SDL_SWSURFACE))==NULL) {
                fprintf(stderr,"SDL_SetVideoMode error: %s\n",SDL_GetError());
                exit(2);
        }
	SDL_WM_SetCaption("SDLwindow - BMP font test",NULL);

	SDL_SurfFont *font;
	font=SDL_InitSurfFont(scr,fontname);

	while (1) {
		SDL_WriteSurfFont(scr,font,1,1,message);
		
		SDL_Flip(scr);
		
                while (SDL_PollEvent(&event)) {
                        switch (event.type) {
                                case SDL_KEYDOWN:
                                        break;
                                case SDL_KEYUP:
                                        if (event.key.keysym.sym==SDLK_f) {
                                                SDL_WM_ToggleFullScreen(scr);
                                        }
					if (event.key.keysym.sym==SDLK_ESCAPE) {
						quitapp(font);
					}
                                        break;
                                case SDL_QUIT:
					quitapp(font);
					break;
                                       
			}
		}
	}
	return 0;
}
