#include <stdio.h>
#include <math.h>
#include <SDL/SDL.h>

#include "sdlgraph.h"

SDL_Surface *initvideo(int x, int y, int d) {
	SDL_Surface *scr;
	if (SDL_Init(SDL_INIT_VIDEO)<0) {
		fprintf(stderr,"Unable to init SDL:%s\n",SDL_GetError());
		return NULL;
	}
	atexit(SDL_Quit);
	scr=SDL_SetVideoMode(x,y,d,SDL_SWSURFACE);
	if (scr==NULL) {
		fprintf(stderr,"Unable to set videomode: %s\n",SDL_GetError());
		return NULL;
	}
	return scr;
}

int closevideo() {
	SDL_Quit();
}

int putpixel_f(SDL_Surface *scr,int x, int y, int c) {
	((unsigned int *)scr->pixels)[y*(scr->pitch/4)+x]=c;
	return 0;
}

int putpixel(SDL_Surface *scr, int x, int y, int c) {
	if ((x>1) && (x<scr->w) && (y>1) && (y<scr->h)) {
		((unsigned int *)scr->pixels)[y*(scr->pitch/4)+x]=c;
	}
	return 0;
}
	
/*int clear(SDL_Surface *scr);*/
int circle(SDL_Surface *scr, int x, int y, int r, int c) {
	int xs,ys,da,db,s;
	if (r==0)
		return 0;
	if (r==1) {
		putpixel(scr,x,y,c);
		return 0;
	}
        xs=0;
	ys=r;
	do {
		da=pow(xs+1,2)+pow(ys,2)-pow(r,2);
		db=pow(xs+1,2)+pow(ys-1,2)-pow(r,2);
		s=da+db;
		xs+=1;
		if (s>0)
			ys-=1;
		putpixel(scr,x+xs-1,y-ys+1,c);
                putpixel(scr,x-xs+1,y-ys+1,c);
                putpixel(scr,x+ys-1,y-xs+1,c);
                putpixel(scr,x-ys+1,y-xs+1,c);
                putpixel(scr,x+xs-1,y+ys-1,c);
                putpixel(scr,x-xs+1,y+ys-1,c);
                putpixel(scr,x+ys-1,y+xs-1,c);
                putpixel(scr,x-ys+1,y+xs-1,c);
	} while (xs<=ys);
	return 0;
}
		
int line(SDL_Surface *scr, int x1, int y1, int x2, int y2, int c) {
	int d,dx,dy,ai,bi,xi,yi,x,y;
        d=dx=dy=ai=bi=xi=yi=0;
        x=x1;
        y=y1;
        if (x<x2) {
                xi=1;
                dx=x2-x;
        }
        else {
                xi=-1;
                dx=x-x2;
        }
        if (y<y2) {
                yi=1;
                dy=y2-y;
        }
        else {
                yi=-1;
                dy=y-y2;
        }
        putpixel(scr,x,y,c);
        if (dx>dy) {
                ai=(dy-dx)*2;
                bi=dy*2;
                d=bi-dx;
                do {
                        if (d>=0) {
                                y+=yi;
                                d+=ai;
                        }
                        else {
                                d+=bi;
                        }
                        x+=xi;
                        putpixel(scr,x,y,c);
                } while (x!=x2);
        }
        else {
                ai=(dx-dy)*2;
                bi=dx*2;
                d=bi-dy;
                do {
                        if (d>=0) {
                                x+=xi;
                                d+=ai;
                        }
                        else {
                                d+=bi;
                        }
                        y+=yi;
                        putpixel(scr,x,y,c);
                } while (y!=y2);
        }
	return 0;
}
int bar(SDL_Surface *scr, int x1, int y1, int x2, int y2, int c) {
	line(scr,x1,y1,x2,y1,c);
	line(scr,x1,y1,x1,y2,c);
	line(scr,x1,y2,x2,y2,c);
	line(scr,x2,y1,x2,y2,c);
	return 0;
}
int fillbar(SDL_Surface *scr, int x1, int y1, int x2, int y2, int c) {
	int y;
        for (y=y1;y<y2;y++) {
                line(scr,x1,y,x2,y,c);
	}
	return 0;
}

int disk(SDL_Surface *scr, int xc, int yc, int a, int b, int c) {
	int  x, y;
	long int aa, aa2,bb, bb2,d, dx, dy;

	x= 0;
	y= b;
	
	aa=a*a;
	aa2=2*aa;
	bb=b*b;
	bb2=2*bb;
	d=bb-aa*b+aa/4;
	dx=0;
	dy=aa2*b; 
	line(scr,xc,yc-y,xc,yc+y,c);
 
	while (dx<dy) {
		if (d>0) {
			y--;
			dy-=aa2;
			d-=dy;
		}
		x++;
		dx+=bb2;
		d+=bb+dx;
		line(scr,xc-x,yc-y,xc-x,yc+y,c);
		line(scr,xc+x,yc-y,xc+x,yc+y,c);
	}
	d+=(3*(aa-bb)/2-(dx+dy))/2;
	while (y>=0) {
		if (d<0) {
			x++;
			dx+=bb2;
			d+=bb+dx;
			line(scr,xc-x,yc-y,xc-x,yc+y,c);
			line(scr,xc+x,yc-y,xc+x,yc+y,c);
		}
		y--;
		dy-=aa2;
		d+=aa-dy;
	}
	return 0;
}
int flip(SDL_Surface *scr) {
	SDL_Flip(scr);
}
