#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <SDL/SDL.h>
#include <math.h>

#include "sdlgraph.h"

#define WIDTH 640
#define HEIGHT 480

struct point3d {
	float x,y,z;
};

int putpixel3d(SDL_Surface *scr, struct point3d p, int c) {
	int nx=(p.x*512)/(p.z+512)+WIDTH/2;
	int ny=(p.y*512)/(p.z+512)+HEIGHT/2;
	if ((nx>0) && (nx<WIDTH) && (ny>0) && (ny<HEIGHT)) {
		((unsigned int *)scr->pixels)[ny*(scr->pitch/4)+nx]=c;
	}
	return 0;
}

int line3d(SDL_Surface *scr, struct point3d p, struct point3d q, int c) {
	int px=(p.x*512)/(p.z+512)+WIDTH/2;
	int py=(p.y*512)/(p.z+512)+HEIGHT/2;
	int qx=(q.x*512)/(q.z+512)+WIDTH/2;
	int qy=(q.y*512)/(q.z+512)+HEIGHT/2;
	line(scr,px,py,qx,qy,c);
	return 0;
}
	
int draw_cube(SDL_Surface *scr, struct point3d cube[7],int c) {
	line3d(scr,cube[0],cube[1],c);
	line3d(scr,cube[0],cube[2],c);
	line3d(scr,cube[0],cube[4],c);
	line3d(scr,cube[1],cube[3],c);
	line3d(scr,cube[1],cube[5],c);
	line3d(scr,cube[2],cube[3],c);
	line3d(scr,cube[2],cube[6],c);
	line3d(scr,cube[3],cube[7],c);
	line3d(scr,cube[4],cube[5],c);
	line3d(scr,cube[4],cube[6],c);
	line3d(scr,cube[5],cube[7],c);
	line3d(scr,cube[6],cube[7],c);
	return 0;
}

struct point3d rotatex(struct point3d p,double angle) {
	struct point3d o;
	double sa=sin(angle),ca=cos(angle);
	o.x=p.x;
        o.y=p.y*ca-p.z*sa;
        o.z=p.y*sa+p.z*ca;
        return o;
}
struct point3d rotatey(struct point3d p,double angle) {
	struct point3d o;
	double sa=sin(angle),ca=cos(angle);
	o.x=p.z*sa+p.x*ca;
        o.y=p.y;
        o.z=p.z*ca-p.x*sa;
        return o;
}
struct point3d rotatez(struct point3d p,double angle) {
	struct point3d o;
	double sa=sin(angle),ca=cos(angle);
	o.x=p.x*ca-p.y*sa;
        o.y=p.x*sa+p.y*ca;
        o.z=p.z;
        return o;
}

struct point3d *definecube(int size) {
	struct point3d *cube;
	cube=(struct point3d *)malloc(sizeof(struct point3d)*8);
        cube[0].x=-size;cube[0].y= size;cube[0].z=-size;
        cube[1].x=-size;cube[1].y=-size;cube[1].z=-size;
        cube[2].x= size;cube[2].y= size;cube[2].z=-size;
        cube[3].x= size;cube[3].y=-size;cube[3].z=-size;
        cube[4].x=-size;cube[4].y= size;cube[4].z= size;
        cube[5].x=-size;cube[5].y=-size;cube[5].z= size;
        cube[6].x= size;cube[6].y= size;cube[6].z= size;
        cube[7].x= size;cube[7].y=-size;cube[7].z= size;
	return cube;
}

int main() {
	SDL_Surface *scr;
	SDL_Event event;

	if (SDL_Init(SDL_INIT_VIDEO)<0) {
                fprintf(stderr,"SDL_Init error: %s\n",SDL_GetError());
                exit(2);
        }
        atexit(SDL_Quit);
        if ((scr=SDL_SetVideoMode(WIDTH,HEIGHT,32,SDL_SWSURFACE))==NULL) {
                fprintf(stderr,"SDL_SetVideoMode error: %s\n",SDL_GetError());
                exit(2);
        }
        SDL_WM_SetCaption("SDLwindow - Plotting test",NULL);
	
	struct point3d tmp,*cube;
	cube=definecube(150);

	int i;
	srand(time(NULL));
	while (1) {
		draw_cube(scr,cube,0x000000);
		for (i=0;i<8;i++) {
			cube[i]=rotatex(cube[i],0.1);
			cube[i]=rotatez(cube[i],0.04);
		}
		draw_cube(scr,cube,rand()%256*65536+rand()%256*256+rand()%256);
		
                while (SDL_PollEvent(&event)) {
                        switch (event.type) {
                                case SDL_KEYDOWN:
                                        break;
                                case SDL_KEYUP:
                                        if (event.key.keysym.sym==SDLK_ESCAPE) {
                                                SDL_Quit();
						return 0;
                                        }
                                        if (event.key.keysym.sym==SDLK_f) {
                                                SDL_WM_ToggleFullScreen(scr);
                                        }
                                        break;
                                case SDL_QUIT:
					SDL_Quit();
                                        return 0;
                        }
                }
		flip(scr);
	}
	return 0;
}
